import os
from cxpm.config import config
from cxpm.tools import download, extract
from cxpm.cmake import CMake

usage = """
find_package(concurrentqueue CONFIG REQUIRED)
target_link_libraries(<target> concurrentqueue::concurrentqueue)
"""

url_template = (
    "https://github.com/glfw/glfw/releases/download/{version}/glfw-{version}.zip"
)


class Builder:
    def __init__(self, version="3.3.9"):
        self.name = "glfw"
        self.version = version
        self.url = url_template.format(version=self.version)
        self.install_dir = f"{config.install_dir}/{self.name}"
        self.usage = usage

    def source(self):
        file_path = download(self.url)
        self.source_path = extract(file_path)

    def build(self):
        self.cmake = CMake()
        self.cmake.configure(self.source_path)
        self.cmake.build()

    def install(self):
        self.cmake.install(self.install_dir)



import sys
from typing import List, Optional
import argparse
import importlib

import cxpm.config


def main(args: (Optional[List[str]]) = None) -> int:
    cxpm.config.init()

    parser = argparse.ArgumentParser(
        description="Install packages.", usage="cxpm install <package>"
    )
    parser.add_argument("command", help="The command to execute.")
    parser.add_argument("package", help="The package to install.")
    args = parser.parse_args(args)

    if args.command != "install":
        parser.print_help()
        return 1

    module = importlib.import_module(f"cxpm_builders.{args.package}.builder")
    Builder = getattr(module, "Builder")

    builder = Builder()
    builder.source()
    builder.build()
    builder.install()

    print(f"Installed {builder.name} {builder.version}")
    print(f"Usage: {builder.usage}")
    return 0


if __name__ == "__main__":
    sys.exit(main(sys.argv[1:]))



import subprocess
import threading
import locale


def print_text(text):
    try:
        print(text.strip().decode(locale.getpreferredencoding()))
    except UnicodeDecodeError:
        print(text.strip().decode(locale.getpreferredencoding(), "ignore"))


def read_pipe(process_pipe, echo=True):
    for line in iter(process_pipe.readline, b""):
        print_text(line)


def cmd(command, echo=True):
    if echo:
        print(f"\033[92m cmd: {command} \033[0m")

    process = subprocess.Popen(
        command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE
    )

    stdout_thread = threading.Thread(target=read_pipe, args=(process.stdout, echo))
    stderr_thread = threading.Thread(target=read_pipe, args=(process.stderr, echo))
    stdout_thread.start()
    stderr_thread.start()

    process.wait()
    stdout_thread.join()
    stderr_thread.join()

    if process.returncode != 0:
        raise Exception(f"Failed Return Code: {process.returncode}")


def script(script, echo=True, shell=True):
    pass



import os
import configparser

class Config:
    def __init__(self):
        conf = configparser.ConfigParser()
        default_config_path = os.path.join(os.path.dirname(__file__), 'config.ini')
        user_config_path = os.path.expanduser("~/.cxpm/config.ini")
        conf.read([default_config_path, user_config_path])

        self.catch_dir = conf['global']['cache_dir']
        self.install_dir = conf['global']['install_dir']
        self.download_dir = f"{self.catch_dir}/download"
        self.build_dir = f"{self.catch_dir}/build"
        self.toolchain = "vs2019"

config = None

def init():
    global config
    config = Config()


from cxpm.config import config
from cxpm.tools import download, extract
from cxpm.cmake import CMake

usage = """
find_package(Eigen3 CONFIG REQUIRED)
target_link_libraries(<target> Eigen3::Eigen)
"""

url_template = (
    "https://gitlab.com/libeigen/eigen/-/archive/{version}/eigen-{version}.zip"
)


class Builder:
    def __init__(self, version="3.4.0"):
        self.name = "eigen3"
        self.version = version
        self.url = url_template.format(version=self.version)
        self.install_dir = f"{config.install_dir}/{self.name}"
        self.usage = usage

    def source(self):
        file_path = download(self.url)
        self.source_dir = extract(file_path)

    def build(self):
        self.cmake = CMake()
        self.cmake.configure(self.source_dir)

    def install(self):
        self.cmake.install(self.install_dir)



import os
from pathlib import Path

from cxpm.config import config
from cxpm.tools import download, extract, copy
from cxpm.cmake import CMake

usage = """
find_package(asio CONFIG REQUIRED)
target_link_libraries(<target> asio)
"""

url_template = (
    "https://codeload.github.com/chriskohlhoff/asio/zip/refs/tags/asio-{version}"
)

class Builder:
    def __init__(self, version="1.29.0"):
        self.name = "asio"
        self.version = version
        self.url = url_template.format(version=self.version.replace('.', '-'))
        self.install_dir = f"{config.install_dir}/{self.name}"
        self.usage = usage

    def source(self):
        file_path = download(self.url)
        self.source_path = extract(file_path)

    def build(self):
        current_script_path = os.path.dirname(os.path.realpath(__file__))
        cmake_scripts = Path(current_script_path) / 'cmake' / '*'
        copy(cmake_scripts, self.source_path)

        self.cmake = CMake()
        self.cmake.configure(self.source_path)

    def install(self):
        self.cmake.install(self.install_dir)



import os
from .tools import cmd

from .config import config

BUILD_DIR = config.build_dir


def clone(url, path=BUILD_DIR):
    project_name = os.path.splitext(os.path.basename(url))[0]
    project_path = os.path.join(path, project_name)
    cmd(f"git clone --depth 1 {url} {project_path}")
    return project_path



import os
from pathlib import Path

from .config import config
from .shell import cmd

# vcvarsall_path = r"C:\Program Files\Microsoft Visual Studio\2022\Professional\VC\Auxiliary\Build\vcvarsall.bat"
# set_vs_env = f'call "{vcvarsall_path}" x64'
# toolset_setenv = {
#     "auto": 'echo "auto"',
#     "vs2019": f'call "C:\\Program Files\\Microsoft Visual Studio\\2022\\Professional\\VC\\Auxiliary\\Build\\vcvarsall.bat" x64',
# }

TOOLCHAIN = config.toolchain
BUILD_TYPES = ["Debug", "Release"]
TOOLCHAIN_COMMANDS = {
    "vs2019": f'call "C:\\Program Files\\Microsoft Visual Studio\\2022\\Professional\\VC\\Auxiliary\\Build\\vcvarsall.bat" x64'
}


class CMake:
    def configure(self, source_dir, cache_variables={}):
        self.setenv = TOOLCHAIN_COMMANDS[TOOLCHAIN]
        self.source_dir = Path(source_dir)
        self.build_dir = Path(self.source_dir) / "build"
        os.makedirs(self.build_dir, exist_ok=True)

        cache_flags = ""
        for key, value in cache_variables.items():
            cache_flags += f"-D{key}={value} "

        for type in BUILD_TYPES:
            cache_flags += f"-DCMAKE_BUILD_TYPE:STRING={type}"
            cmd(
                f"{self.setenv} && cmake -S{self.source_dir} -B{self.build_dir}/{type} -G Ninja {cache_flags}"
            )

    def build(self):
        for type in BUILD_TYPES:
            cmd(f"{self.setenv} && cmake --build {self.build_dir}/{type}")

    def install(self, install_dir):
        for type in BUILD_TYPES:
            cmd(f"cmake --install {self.build_dir}/{type} --prefix {install_dir}")

    # def configure_msvc(self):
    #     vcvarsall_path = r"C:\Program Files\Microsoft Visual Studio\2022\Professional\VC\Auxiliary\Build\vcvarsall.bat"
    #     command = f'call "{vcvarsall_path}" x64 && cmake -S{self.source_path} -B{self.build_path} -G Ninja'
    #     cmd(command)



import subprocess
import sys
import os
import zipfile
from pathlib import Path
import urllib.request
import http.client
import shutil
import cgi
import glob
import locale
import select

from .config import config

DOWNLOAD_DIR = config.download_dir
BUILD_DIR = config.build_dir
INSTALL_DIR = config.install_dir


def download(url, filename=None, download_dir=DOWNLOAD_DIR):
    response = urllib.request.urlopen(url)

    # 如果没有提供文件名，尝试从Content-Disposition头部获取
    if filename is None:
        cd = response.headers.get("content-disposition")
        if cd:
            value, params = cgi.parse_header(cd)
            filename = params.get("filename")

        # 如果Content-Disposition头部不存在或者没有文件名，使用URL的路径部分
        if filename is None:
            filename = os.path.basename(urllib.parse.urlsplit(url).path)

    file_path = os.path.join(download_dir, filename)
    temp_file_path = f"{file_path}.download"

    # 如果文件已经存在，不再下载
    if os.path.exists(file_path):
        print(f"File {file_path} already exists. Skipping download.")
        return file_path

    os.makedirs(download_dir, exist_ok=True)
    try:
        with open(temp_file_path, "wb") as file:
            file.write(response.read())
    except http.client.IncompleteRead as e:
        print(
            f"Warning: Only {len(e.partial)} out of {e.expected} bytes received. The file might be incomplete."
        )

    # 重命名临时文件（移除.download后缀）
    os.rename(temp_file_path, file_path)

    return file_path


def get_zip_root(file_path):
    with zipfile.ZipFile(file_path, "r") as zip_ref:
        names = zip_ref.namelist()

    root_dirs = {name.split("/")[0] for name in names}

    if len(root_dirs) == 1:
        return list(root_dirs)[0]
    else:
        return None


def extract(file_path, extract_path=BUILD_DIR):
    zip_root = get_zip_root(file_path)
    if zip_root is not None:
        # 如果有顶层文件夹，解压到extract_path
        with zipfile.ZipFile(file_path, "r") as zip_ref:
            zip_ref.extractall(extract_path)
        # 返回解压后的文件夹路径
        return str(Path(extract_path) / zip_root)
    else:
        # 如果没有顶层文件夹，根据zip包的名称（去掉后缀）创建新文件夹
        zip_name = Path(file_path).stem
        new_dir = Path(extract_path) / zip_name
        new_dir.mkdir(parents=True, exist_ok=True)
        with zipfile.ZipFile(file_path, "r") as zip_ref:
            zip_ref.extractall(new_dir)
        # 返回新创建的文件夹路径
        return str(new_dir)


def copy(src_path, dest_path):
    if os.path.isfile(src_path):
        shutil.copy(src_path, dest_path)
    elif os.path.isdir(src_path):
        dest_dir = os.path.join(dest_path, os.path.basename(src_path))
        shutil.copytree(src_path, dest_dir)
    else:
        for path in glob.glob(str(src_path)):
            copy(path, dest_path)
